package com.github.polpetta.mezzotre.route;

import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.mezzotre.orm.model.query.QTgChat;
import com.github.polpetta.mezzotre.telegram.command.Router;
import com.github.polpetta.types.json.ChatContext;
import com.google.gson.Gson;
import com.pengrad.telegrambot.model.Message;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import io.jooby.Context;
import io.jooby.MediaType;
import io.jooby.annotations.POST;
import io.jooby.annotations.Path;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.parameters.RequestBody;
import io.swagger.v3.oas.annotations.tags.Tag;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import javax.inject.Inject;
import javax.inject.Named;
import org.slf4j.Logger;

@Tag(name = "Telegram", description = "Telegram webhook endpoint")
@Path("/" + Constants.API + "/" + Telegram.ENDPOINT)
public class Telegram {

  static final String ENDPOINT = "tg";
  private final Logger log;
  private final Gson gson;
  private final Executor completableFutureThreadPool;
  private final Router router;

  @Inject
  public Telegram(
      Logger log,
      Gson gson,
      @Named("eventThreadPool") Executor completableFutureThreadPool,
      Router router) {
    this.log = log;
    this.gson = gson;
    this.completableFutureThreadPool = completableFutureThreadPool;
    this.router = router;
  }

  @Operation(
      summary = "Telegram webhook entrypoint",
      description =
          "Telegram servers will call this endpoint to send updates about incoming messages",
      tags = "telegram",
      requestBody = @RequestBody(required = true))
  @POST
  public CompletableFuture<String> incomingUpdate(Context context, Update update) {
    return CompletableFuture.supplyAsync(
            () -> {
              context.setResponseType(MediaType.JSON);
              log.trace(gson.toJson(update));

              final Message message = update.message();
              return new QTgChat()
                  .id
                  .eq(message.chat().id())
                  .findOneOrEmpty()
                  .map(
                      u -> {
                        log.debug(
                            "Telegram chat " + u.getId() + " already registered in the database");
                        return u;
                      })
                  .orElseGet(
                      () -> {
                        final TgChat newTgChat = new TgChat(message.chat().id(), new ChatContext());
                        newTgChat.save();
                        log.trace(
                            "New Telegram chat " + newTgChat.getId() + " added into the database");
                        return newTgChat;
                      });
            },
            completableFutureThreadPool)
        .thenComposeAsync(tgChat -> router.process(tgChat, update), completableFutureThreadPool)
        // See https://core.telegram.org/bots/faq#how-can-i-make-requests-in-response-to-updates
        .thenApply(
            tgResponse -> {
              final String response = tgResponse.map(BaseRequest::toWebhookResponse).orElse("{}");
              log.trace("About to send back " + response);
              return response;
            });
  }
}
