package com.github.polpetta.mezzotre.telegram.command;

import com.github.polpetta.mezzotre.i18n.LocalizedMessageFactory;
import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.types.json.ChatContext;
import com.google.inject.Singleton;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.model.request.ParseMode;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.SendMessage;
import io.vavr.control.Try;
import java.nio.charset.StandardCharsets;
import java.util.Locale;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import javax.inject.Inject;
import javax.inject.Named;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.tools.ToolManager;
import org.apache.velocity.util.StringBuilderWriter;
import org.slf4j.Logger;

/**
 * This {@link Executor} has the goal to greet a user that typed {@code /start} to the bot.
 *
 * @author Davide Polonio
 * @since 1.0
 */
@Singleton
public class Start implements Executor {

  private final java.util.concurrent.Executor threadPool;
  private final Logger log;
  private final LocalizedMessageFactory localizedMessageFactory;

  @Inject
  public Start(
      LocalizedMessageFactory localizedMessageFactory,
      @Named("eventThreadPool") java.util.concurrent.Executor threadPool,
      Logger log) {
    this.localizedMessageFactory = localizedMessageFactory;
    this.threadPool = threadPool;
    this.log = log;
  }

  @Override
  public String getTriggerKeyword() {
    return "/start";
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(TgChat chat, Update update) {
    return CompletableFuture.supplyAsync(
        () -> {
          /*
          Steps:
            1 - Load the template and generate the message with the defined locale
            2 - Update the chat context
            3 - Reply to Telegram
           */
          final String message =
              Try.of(
                      () -> {
                        final Locale locale = Locale.forLanguageTag(chat.getLocale());
                        final ToolManager toolContext = localizedMessageFactory.create(locale);
                        final VelocityContext context =
                            new VelocityContext(toolContext.createContext());
                        context.put("firstName", update.message().chat().firstName());
                        context.put("programName", "Mezzotre");

                        final StringBuilder content = new StringBuilder();
                        final StringBuilderWriter stringBuilderWriter =
                            new StringBuilderWriter(content);

                        toolContext
                            .getVelocityEngine()
                            .mergeTemplate(
                                "template/command/start.vm",
                                StandardCharsets.UTF_8.name(),
                                context,
                                stringBuilderWriter);

                        stringBuilderWriter.close();
                        return content.toString();
                      })
                  .get();
          log.trace("Start command - message to send back: " + message);

          final ChatContext chatContext = chat.getChatContext();
          chatContext.setLastMessageSentId(update.message().messageId());
          chatContext.setStage(getTriggerKeyword());
          chatContext.setStep(0);
          chatContext.setPreviousMessageUnixTimestampInSeconds(update.message().date());
          chat.setChatContext(chatContext);
          chat.save();

          return Optional.of(new SendMessage(chat.getId(), message).parseMode(ParseMode.Markdown));
        },
        threadPool);
  }
}
