package com.github.polpetta.mezzotre.orm.model;

import static org.junit.jupiter.api.Assertions.*;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.polpetta.mezzotre.helper.Loader;
import com.github.polpetta.mezzotre.helper.TestConfig;
import com.github.polpetta.mezzotre.orm.model.query.QTgChat;
import com.github.polpetta.mezzotre.util.Clock;
import com.github.polpetta.types.json.ChatContext;
import io.ebean.Database;
import io.ebean.SqlRow;
import java.sql.Timestamp;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.postgresql.util.PGobject;
import org.testcontainers.containers.PostgreSQLContainer;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

@Tag("slow")
@Tag("database")
@Testcontainers
class TgChatIntegrationTest {

  private static ObjectMapper objectMapper;

  @Container
  private final PostgreSQLContainer<?> postgresServer =
      new PostgreSQLContainer<>(TestConfig.POSTGRES_DOCKER_IMAGE);

  private Database database;

  @BeforeAll
  static void beforeAll() {
    objectMapper = new ObjectMapper();
  }

  @BeforeEach
  void setUp() throws Exception {
    database =
        Loader.connectToDatabase(Loader.loadDefaultEbeanConfigWithPostgresSettings(postgresServer));
  }

  @Test
  void shouldInsertEntryIntoDatabase() throws Exception {

    final ChatContext chatContext =
        new ChatContext.ChatContextBuilder()
            .withStage("/example")
            .withStep(2)
            .withPreviousMessageUnixTimestampInSeconds(42)
            .build();

    final TgChat user = new TgChat(1234L, chatContext, "en");

    user.save();
    final String query = "select * from telegram_chat where id = ?";
    final SqlRow savedChat = database.sqlQuery(query).setParameter(1234L).findOne();

    assertNotNull(savedChat);
    assertEquals(1234L, savedChat.getLong("id"));
    assertEquals("en", savedChat.getString("locale"));
    assertNotNull(savedChat.get("chat_context"));
    assertEquals("jsonb", ((PGobject) savedChat.get("chat_context")).getType());
    assertEquals(
        chatContext,
        objectMapper.readValue(
            ((PGobject) savedChat.get("chat_context")).getValue(), ChatContext.class));
  }

  @Test
  void shouldRetrieveEntryInDatabase() throws Exception {

    final Timestamp timestampFromUnixEpoch = Clock.getTimestampFromUnixEpoch(1L);
    final ChatContext chatContext =
        new ChatContext.ChatContextBuilder()
            .withStage("/start")
            .withStep(1)
            .withPreviousMessageUnixTimestampInSeconds(42)
            .build();

    final String insertQuery = "insert into telegram_chat values (?, ?::jsonb, ?, ?, ?)";
    final int affectedRows =
        database
            .sqlUpdate(insertQuery)
            .setParameter(1234L)
            .setParameter(objectMapper.writeValueAsString(chatContext))
            .setParameter("en-US")
            .setParameter(timestampFromUnixEpoch)
            .setParameter(timestampFromUnixEpoch)
            .execute();

    assertEquals(1, affectedRows);

    final TgChat got = new QTgChat().id.eq(1234L).findOne();
    assertNotNull(got);
    final ChatContext gotChatContext = got.getChatContext();
    assertNotNull(gotChatContext);
    assertEquals("/start", gotChatContext.getStage());
    assertEquals(1, gotChatContext.getStep());
    assertEquals(42, gotChatContext.getPreviousMessageUnixTimestampInSeconds());
  }
}
