package com.github.polpetta.mezzotre.telegram.command;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import com.github.polpetta.mezzotre.i18n.LocalizedMessageFactory;
import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.types.json.ChatContext;
import com.google.gson.Gson;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.SendMessage;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.parallel.Execution;
import org.junit.jupiter.api.parallel.ExecutionMode;
import org.slf4j.Logger;

@Tag("velocity")
@Execution(ExecutionMode.CONCURRENT)
class StartTest {

  private VelocityEngine velocityEngine;
  private LocalizedMessageFactory localizedMessageFactory;
  private Start start;
  private static Gson gson;

  @BeforeAll
  static void beforeAll() {
    gson = new Gson();
  }

  @BeforeEach
  void setUp() {
    velocityEngine = new VelocityEngine();
    velocityEngine.setProperty(RuntimeConstants.RESOURCE_LOADERS, "classpath");
    velocityEngine.setProperty(
        "resource.loader.classpath.class", ClasspathResourceLoader.class.getName());
    velocityEngine.init();
    localizedMessageFactory = new LocalizedMessageFactory(velocityEngine);

    final Logger fakeLog = mock(Logger.class);

    start = new Start(localizedMessageFactory, Executors.newSingleThreadExecutor(), fakeLog);
  }

  @Test
  void shouldReceiveHelloIntroduction() throws Exception {
    final TgChat fakeChat = mock(TgChat.class);
    when(fakeChat.getLocale()).thenReturn("en-US");
    when(fakeChat.getChatContext()).thenReturn(new ChatContext());
    when(fakeChat.getId()).thenReturn(1111111L);

    final Update update =
        gson.fromJson(
            "{\n"
                + "\"update_id\":10000,\n"
                + "\"message\":{\n"
                + "  \"date\":1441645532,\n"
                + "  \"chat\":{\n"
                + "     \"last_name\":\"Test Lastname\",\n"
                + "     \"id\":1111111,\n"
                + "     \"type\": \"private\",\n"
                + "     \"first_name\":\"Test Firstname\",\n"
                + "     \"username\":\"Testusername\"\n"
                + "  },\n"
                + "  \"message_id\":1365,\n"
                + "  \"from\":{\n"
                + "     \"last_name\":\"Test Lastname\",\n"
                + "     \"id\":1111111,\n"
                + "     \"first_name\":\"Test Firstname\",\n"
                + "     \"username\":\"Testusername\"\n"
                + "  },\n"
                + "  \"text\":\"/start\"\n"
                + "}\n"
                + "}",
            Update.class);

    final CompletableFuture<Optional<BaseRequest<?, ?>>> gotFuture =
        start.process(fakeChat, update);
    assertDoesNotThrow(() -> gotFuture.get());
    final Optional<BaseRequest<?, ?>> gotMessageOptional = gotFuture.get();
    assertDoesNotThrow(gotMessageOptional::get);
    final BaseRequest<?, ?> gotMessage = gotMessageOptional.get();
    assertInstanceOf(SendMessage.class, gotMessage);
    final String message = (String) gotMessage.getParameters().get("text");
    assertEquals(
        "**Hello Test Firstname! \uD83D\uDC4B**\n\n"
            + "This is _Mezzotre_, a simple bot focused on DnD content management! Please start by"
            + " choosing a language down below. \uD83D\uDC47",
        message);
    assertEquals(1111111L, (Long) gotMessage.getParameters().get("chat_id"));
    verify(fakeChat, times(1)).save();
  }

  @Test
  void shouldThrowErrorIfLocaleNonExists() {
    final TgChat fakeChat = mock(TgChat.class);
    // Do not set Locale on purpose
    when(fakeChat.getId()).thenReturn(1111111L);

    final Update update =
        gson.fromJson(
            "{\n"
                + "\"update_id\":10000,\n"
                + "\"message\":{\n"
                + "  \"date\":1441645532,\n"
                + "  \"chat\":{\n"
                + "     \"last_name\":\"Test Lastname\",\n"
                + "     \"id\":1111111,\n"
                + "     \"type\": \"private\",\n"
                + "     \"first_name\":\"Test Firstname\",\n"
                + "     \"username\":\"Testusername\"\n"
                + "  },\n"
                + "  \"message_id\":1365,\n"
                + "  \"from\":{\n"
                + "     \"last_name\":\"Test Lastname\",\n"
                + "     \"id\":1111111,\n"
                + "     \"first_name\":\"Test Firstname\",\n"
                + "     \"username\":\"Testusername\"\n"
                + "  },\n"
                + "  \"text\":\"/start\"\n"
                + "}\n"
                + "}",
            Update.class);

    final CompletableFuture<Optional<BaseRequest<?, ?>>> gotFuture =
        start.process(fakeChat, update);
    assertThrows(ExecutionException.class, gotFuture::get);
  }
}
